@@ -76,7 +76,7 @@ def setup_pytester(pytester, coverage_project):
                     # Hack in in-memory overrides for testing purposes.
                     text = str(coverage_project.config)
                 else:
-                    text = file.read_text()
+                    text = file.read_text(encoding="utf8")
 
                 src = {name: text.splitlines()}
                 pytester.makefile(file.suffix, **src)
@@ -87,7 +87,7 @@ def setup_pytester(pytester, coverage_project):
     test_files = {}
     for file_path in tests_path.iterdir():
         if file_path.name.startswith("test_") and file_path.suffix == ".py":
-            content = file_path.read_text()
+            content = file_path.read_text(encoding="utf8")
             test_files[file_path.name] = content
             num_passes += len(
                 [
@@ -104,7 +104,7 @@ def setup_pytester(pytester, coverage_project):
     # Check for a conftest.py
     conftest = tests_path / "conftest.py"
     if conftest.is_file():
-        pytester.makeconftest(conftest.read_text())
+        pytester.makeconftest(conftest.read_text(encoding="utf8"))
 
     # Returns expected number of passing tests.
     return num_passes, num_failed
@@ -166,7 +166,7 @@ def _assert_coverage(actual: list[str], expected: list[str]):
 
 def _assert_xml(xml_path: Path):
     assert xml_path.is_file()
-    xml = xml_path.read_text()
+    xml = xml_path.read_text(encoding="utf8")
     assert '<?xml version="1.0" ?>' in xml
 
     # Show is valid XML.
@@ -198,7 +198,7 @@ def _assert_xml(xml_path: Path):
 
 
 def _assert_html(index_html: Path):
-    html = index_html.read_text()
+    html = index_html.read_text(encoding="utf8")
     assert html.startswith("<!DOCTYPE html>")
     assert "Generated by Ape Framework" in html
     expected_columns = (
